# Copyright (C) 2006-2014 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

import IN
import socket

from agent.api.remote.base import RemoteManagement
from agent.lib.daemon import delay_cmd
from agent.target.host import retry_cmd


class LinuxRemoteManagement(RemoteManagement):
    """
    Class for remote server management.
    """

    def get_services(self, version=None):
        return {
            'sys_shutdown': self.sys_shutdown,
            'sys_reboot': self.sys_reboot,
            'remote_power_off': self.remote_power_off,
            'remote_power_on': self.remote_power_on,
        }

    def sys_shutdown(self):
        """
        Shutdown the server.

        @raise IModuleException: on failure

        """
        delay_cmd(5, ["shutdown", "-P", "-h", "now", "shutdown requested by OVM"])

    def sys_reboot(self):
        """
        Reboot the server.

        @raise IModuleException: on failure

        """
        delay_cmd(5, ["shutdown", "-r", "now", "reboot requested by OVM"])


    def _ipmi_cmd(self, host, command, username=None, password=None):
        cmd = ["ipmitool", "-I", "lan", "-H", host]
        if username:
            cmd.extend(["-U", username])
        if password:
            cmd.extend(["-P", password])
        cmd.extend(command)
        #
        # It transpires that Dell BMC's are somewhat unreliable
        # (actually it may not just be exclusive to them but the
        # result is that we need to send this command a number of
        # times to make sure it was received)
        # leave in a small interval for the card to wake up and
        # process the command
        #
        try:
            retry_cmd(cmd, interval=5, count=5)
        except Exception:
            # try lanplus
            cmd[2] = 'lanplus'
            try:
                retry_cmd(cmd, interval=5, count=5)
            except Exception:
                raise RuntimeError('Failed to run: ipmitool -I lan/lanplus -H %s %s' % (host, command))


    def _bind_sock_to_device(self, sock, net_device):
        sock.setsockopt(socket.SOL_SOCKET, IN.SO_BINDTODEVICE, net_device)
