# Copyright (C) 2006-2014 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.

import logging
import time

from agent.lib.constants import LOCAL_SERVER_DB
from agent.lib.db import write_item, read_item


STAT_INTERVAL_DEFAULT = 20


def set_statistic_interval(interval):
    write_item(LOCAL_SERVER_DB, "stat_interval", interval)


def get_statistic_interval():
    return read_item(LOCAL_SERVER_DB, "stat_interval") or STAT_INTERVAL_DEFAULT


def sleep_for_statistic_interval():
    elapse = 0
    while True:
        time.sleep(1)
        elapse += 1
        statistic_interval = get_statistic_interval()
        if statistic_interval == -1:
            continue
        if elapse >= statistic_interval:
            break


class StatsCollectorBase(object):
    def __init__(self):
        self._stop = False

    def run(self):
        while not self._stop:
            try:
                self._run()
            except Exception, err:
                logging.exception('Error collecting stats: %s', err)
            sleep_for_statistic_interval()

    def _run(self):
        # to be overridden by subclasses
        pass

    def stop(self):
        self._stop = True
