#!/usr/bin/env python
#
# Copyright (C) 2011-2014 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Utilities for doing wget
"""

import os
import sys
import re
import time

import st_utils

from st_network		import *
from st_misc		import *

####################################################################################################

__all__ = [	'vi_confirm_network_connectivity'
		,	'vi_sanitize_pull_URL'
		,	'vi_pull_node_configuration_file' ]

####################################################################################################
def vi_confirm_network_connectivity( vi_errorAndLog_obj, ip_address ):
	"""
	This routine
	"""
#
	exitStatus	= vi_ping_test( vi_errorAndLog_obj, ip_address, 30 )
	if exitStatus == 0:
		return False

	# Try harder

	interface_DICT	= vi_get_NIC_dict(   vi_errorAndLog_obj, False, True )
	route_LIST		= vi_get_route_list( vi_errorAndLog_obj, True )

	vi_errorAndLog_obj.vi_print_log( "Trouble getting to: '%s'" % ( ip_address ) )
	vi_errorAndLog_obj.vi_print_log( "Network Interfaces:" )
	for interface, nic_state in interface_DICT.iteritems():
		vi_errorAndLog_obj.vi_print_log( "    %-8s -> %s: %-16s %-16s" % ( \
												nic_state.mInterface
											,	nic_state.mMAC
											,	nic_state.mIPv4_addr
											,	nic_state.mIPv4_netmask ) )

	vi_errorAndLog_obj.vi_print_log( "Routes:" )
	for route in route_LIST:
		vi_errorAndLog_obj.vi_print_log( '   ', route )

	time.sleep( 10 )

	exitStatus	= vi_ping_test( vi_errorAndLog_obj, ip_address, 90 )
	if exitStatus == 0:
		vi_errorAndLog_obj.vi_print_log( "ping 90 succeeded" )
		return False

	vi_errorAndLog_obj.vi_print_log( "ping 90 failed" )
	vi_errorAndLog_obj.vi_print_log( "PS:" )

	ps_process_DICT	= vi_get_ps_process_dict( global_errAndLog, True )
	for pid, ps_process in ps_process_DICT.iteritems():
		if ps_process.mSessionID < 2:
			continue
		vi_errorAndLog_obj.vi_print_log( "%6s %6s %10s %s" % ( ps_process.mPercentCPU, ps_process.mPercentMemory, ps_process.mVirtualMemorySize, ' '.join( ps_process.mCmd_LIST ) ) )

	vi_errorAndLog_obj.vi_print_err( "Failed to establish network connectivity to: '%s'" % ( ip_address ) )
	return True

####################################################################################################
def vi_sanitize_pull_URL( vi_errorAndLog_obj, pullURL, primaryIface ):
	"""
	This routine
	"""
#
	vi_errorAndLog_obj.vi_print_dbg( 3, "sanitize_pull_URL( '%s', '%s' )" % ( pullURL, primaryIface ) )

	httpURL_cmpldRegEx	= re.compile( '^http://(\d+.\d+.\d+.\d+)' )
	httpURL_match		= httpURL_cmpldRegEx.match( pullURL )
	if not httpURL_match:
		fileURL_cmpldRegEx	= re.compile( '^file://' )
		if fileURL_cmpldRegEx.match( pullURL ):
			return pullURL
		vi_errorAndLog_obj.vi_print_err( "sanitize_pull_URL( '%s', '%s' ) BAD URL" % ( pullURL, primaryIface ) )
		return None

	if vi_confirm_network_connectivity( vi_errorAndLog_obj, httpURL_match.group(1) ):
		return None		# Pull is doomed

	oldURL_cmpldRegEx	= re.compile( '^(.*)/configure_node$' )
	oldURL_match		= oldURL_cmpldRegEx.match( pullURL )
	if oldURL_match:
		pullURL			= oldURL_match.group(1)

	subAtURL_cmpldRegEx	= re.compile( '^(.*)/@$' )
	subAtURL_match		= subAtURL_cmpldRegEx.match( pullURL )
	if not subAtURL_match:

		# Try pullURL
		cmd_LIST	= [ '/usr/bin/wget', '--spider', '--progress=dot', pullURL ]
		stdout_LIST, stderr_LIST, exitStatus, deltaTime	= st_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST, 30 )
		if exitStatus == 0:
			return pullURL

		vi_errorAndLog_obj.vi_print_err( "Spider check failed for '%s' ( %s ) failed : URL bad" % ( pullURL, primaryIface ) )
		return None		# Pull is doomed

	basePath										= subAtURL_match.group(1) + '/'
	cmd_LIST										= [ '/usr/bin/wget', '--spider', '--progress=dot', basePath ]
	stdout_LIST, stderr_LIST, exitStatus, deltaTime	= st_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST, 30 )
	if exitStatus != 0:
		vi_errorAndLog_obj.vi_print_err( "Spider check failed for base '%s' ( %s ) failed : URL bad '%s'" % ( basePath, primaryIface, pullURL ) )
		return None		# Pull is doomed

	interface_DICT	= vi_get_NIC_dict( vi_errorAndLog_obj, False, False )
	nic_state		= interface_DICT.get( primaryIface, None )
	if not nic_state:
		vi_errorAndLog_obj.vi_print_err( "sanitize_pull_URL( '%s', '%s' ) :: primaryIface does not exist" % ( pullURL, primaryIface ) )
		vi_errorAndLog_obj.vi_print_log( "interface_DICT:", interface_DICT )
		return None		# Pull is doomed

	# Try node_<primaryIP>
	final_pullURL									= "%s/node_%s" % ( subAtURL_match.group(1), nic_state.mIPv4_addr )
	cmd_LIST										= [ '/usr/bin/wget', '--spider', '--progress=dot', final_pullURL ]
	stdout_LIST, stderr_LIST, exitStatus, deltaTime	= st_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST, 30 )
	if exitStatus == 0:
		return final_pullURL

	vi_errorAndLog_obj.vi_print_dbg( 3, "sanitize_pull_URL( '%s', '%s' ) :: try '%s' failed" % ( pullURL, primaryIface, final_pullURL ) )

	# Try node_default
	final_pullURL									= "%s/node_default" % ( subAtURL_match.group(1) )
	cmd_LIST										= [ '/usr/bin/wget', '--spider', '--progress=dot', final_pullURL ]
	stdout_LIST, stderr_LIST, exitStatus, deltaTime	= st_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST, 30 )
	if exitStatus == 0:
		return final_pullURL

	vi_errorAndLog_obj.vi_print_dbg( 3, "sanitize_pull_URL( '%s', '%s' ) :: default '%s' failed" % ( pullURL, primaryIface, final_pullURL ) )

	vi_errorAndLog_obj.vi_print_err( "Spider check failed for '%s' ( %s ) failed : URL bad" % ( pullURL, primaryIface ) )
	return None		# Pull is doomed

####################################################################################################
def vi_pull_node_configuration_file( vi_errorAndLog_obj, pullURL, localConfigDirPath, fileName, isOptionalFile ):
	"""
	This routine
	"""
#
	srcPath	=            pullURL + '/' + fileName
	dstPath	= localConfigDirPath + '/' + fileName

	if os.path.exists( dstPath ):
		os.unlink( dstPath )

	if not isOptionalFile:
		vi_errorAndLog_obj.vi_print_log( "Downloading:    %s" % ( srcPath ) )

	cmd_LIST	=  [ '/usr/bin/wget', '-r', '--no-directories', '--directory-prefix', localConfigDirPath, '--progress=dot', srcPath ]	# -r for overwrite

#	vi_errorAndLog_obj.vi_print_dbg( 0, "CMD: %s" % ( ' '.join( cmd_LIST ) ) )

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST, 30 )

	output_str	= ''

	if len(stdout_LIST):
		stdout_LIST	= map( str.strip, stdout_LIST )
		for idx in reversed(range(len(stdout_LIST))):
			if len(stdout_LIST[idx]) == 0:
				del( stdout_LIST[idx] )
		if len(stdout_LIST):
			output_str	= '\n'.join( stdout_LIST ) + '\n'

	if len(stderr_LIST):
		stderr_LIST	= map( str.strip, stderr_LIST )
		for idx in reversed(range(len(stderr_LIST))):
			if len(stderr_LIST[idx]) == 0:
				del( stderr_LIST[idx] )
		if len(stderr_LIST):
			output_str	+= '\n'.join( stderr_LIST )

	if exitStatus != 0 or not os.path.exists( dstPath ):
		if not isOptionalFile:
			vi_errorAndLog_obj.vi_print_err( "Failed to pull: '%s'\n" % ( srcPath ) + '\n' + output_str )
#		vi_errorAndLog_obj.vi_print_log( "Optional pull failed: %s" % ( dstPath ) )
		return None

	vi_errorAndLog_obj.vi_print_log( "Pull succeeded: %s" % ( dstPath ) )
	vi_errorAndLog_obj.vi_print_alt( output_str )

	return dstPath

####################################################################################################
# END
