#!/usr/bin/env python
#
# Copyright (C) 2011-2014 Oracle. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, version 2.  This program is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the GNU
# General Public License along with this program; if not, write to the Free
# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 021110-1307, USA.
####################################################################################################
#
#	Author:		Joshua Nicholas
#
"""Utilities for dealing with LVM
"""

import st_utils

import re
import traceback

from deviceMapper	import *

####################################################################################################

__all__ = [	'PV_Info_OBJ'
		,	'VG_Info_OBJ'
		,	'LV_Info_OBJ'
		,	'Basic_VG_Info_OBJ'
		,	'get_pv_wwid_from_dm_minor'
		,	'add_lvm_info'
		,	'get_lvm_info'
		,	'get_basic_vg_info'
		,	'is_vdisk_lv_name'
		,	'do_dom0_memory_safe_vgchange' ]

####################################################################################################

class PV_Info_OBJ:

	def __init__(self):
		self.mWWID		= None
		self.mUUID		= None
		self.mVGname	= None

	def __repr__(self):
		return "mWWID %s mUUID %s mVGname %s" % (
						self.mWWID
					,	self.mUUID
					,	self.mVGname )

	def __cmp__( self, other ):
		if not isinstance( other, PV_Info_OBJ ):
			return -1
		return cmp( self.mUUID, other.mUUID )

class VG_Info_OBJ:

	def __init__(self):
		self.mName				= None
		self.mUUID				= None
		self.mSizeInBytes		= None
		self.mFreeSizeInBytes	= None
		self.mPVinfo_wwid_DICT	= None # PV_Info_OBJ[ mWWID ]
		self.mLVinfo_name_DICT	= None # LV_Info_OBJ[ mName ]
		self.mNumOfLVsnapShots	= None

	def __repr__(self):
		return "mName %s mUUID %s mSizeInBytes %s mFreeSizeInBytes %s mPVinfo_wwid_DICT %s mLVinfo_name_DICT %s mNumOfLVsnapShots %s" % (
						self.mName
					,	self.mUUID
					,	self.mSizeInBytes
					,	self.mFreeSizeInBytes
					,	self.mPVinfo_wwid_DICT
					,	self.mLVinfo_name_DICT
					,	self.mNumOfLVsnapShots )

	def __cmp__( self, other ):
		if not isinstance( other, VG_Info_OBJ ):
			return -1
		return cmp( self.mName, other.mName )

class LV_Info_OBJ:

	def __init__(self):
		self.mName           = None
		self.mUUID           = None
		self.mVGname         = None
		self.mVGuuid         = None
		self.mSizeInBytes    = None
		self.mAttr           = None
		self.mOrigin         = None
		self.mSnapPercent    = None

	def __repr__(self):
		return "mName %s mUUID %s mVGname %s mVGuuid %s mSizeInBytes %s mAttr %s mOrigin %s mSnapPercent %s" % (
						self.mName
					,	self.mUUID
					,	self.mVGname
					,	self.mVGuuid
					,	self.mSizeInBytes
					,	self.mAttr
					,	self.mOrigin
					,	self.mSnapPercent )

class Basic_VG_Info_OBJ:

	def __init__(self):
		self.mName				= None
		self.mUUID				= None
		self.mSizeInBytes		= None
		self.mFreeSizeInBytes	= None
		self.mNumOfPVs			= None
		self.mNumOfLVs			= None
		self.mNumOfLVsnapShots	= None

	def __repr__(self):
		return "mName %s mUUID %s mSizeInBytes %s mFreeSizeInBytes %s mNumOfPVs %s mNumOfLVs %s mNumOfLVsnapShots %s" % (
						self.mName
					,	self.mUUID
					,	self.mSizeInBytes
					,	self.mFreeSizeInBytes
					,	self.mNumOfPVs
					,	self.mNumOfLVs
					,	self.mNumOfLVsnapShots )

####################################################################################################

global__cached__pvInfo_uuid_DICT		= None
global__cached__vgInfo_uuid_DICT		= None
global__cached__basicVGinfo_uuid_DICT	= None

####################################################################################################
def get_pv_wwid_from_dm_minor( vi_errorAndLog_obj, dm_minor ):
	"""
	This routine
	"""
#
	device_mapper_major_num	= get_device_mapper_major_num( vi_errorAndLog_obj )

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, "devmap_name %s %s" % ( device_mapper_major_num, dm_minor ) )
	if len(stdout_LIST) > 0:
		return stdout_LIST[0].strip()

	return None

####################################################################################################
def add_lvm_info( vi_errorAndLog_obj, lookForPVsToAdd, pvInfo_uuid_DICT, vgInfo_uuid_DICT ):
	"""
	This routine
	"""
#
	if lookForPVsToAdd:
		pvscan_line_cmpldRegEx   = re.compile( '^\s*PV\s+(\/dev\/\S+)\s+with UUID\s+(\S+)\s+(.*)lvm2\s+\[.*]$' )
		optional_vg_cmpldRegEx   = re.compile( '^VG\s+(\S+)\s+$' )
		pv_name_short_cmpldRegEx = re.compile( '^\/dev\/dm-(\d+)$' )
		pv_name_long_cmpldRegEx  = re.compile( '^\/dev\/mapper\/(\S+)$' )

		stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, 'pvscan --uuid' )
		del( stderr_LIST )
		pvs_lines_LIST = map( str.strip, stdout_LIST )
		del( stdout_LIST )
		pvs_line = ''
		for pvs_line in pvs_lines_LIST:
			if len( pvs_line ) < 1:
				continue
			try:
				pvscan_line_match = pvscan_line_cmpldRegEx.match( pvs_line )
				if not pvscan_line_match:
					continue

				pv_name = pvscan_line_match.group( 1 )
				pv_uuid = pvscan_line_match.group( 2 )
				if pvInfo_uuid_DICT.has_key( pv_uuid ): # Already exists
					continue

				pv_name_match = pv_name_long_cmpldRegEx.match( pv_name )
				if pv_name_match:
					pv_wwid = pv_name_match.group( 1 )
				else:
					pv_name_match = pv_name_short_cmpldRegEx.match( pv_name )
					if not pv_name_match:
#						vi_errorAndLog_obj.vi_croak( "bad pv_name in pvscan : [%s] %s" % ( pv_name, pvs_line ) )
						vi_errorAndLog_obj.vi_print_err( "bad pv_name in pvscan : [%s] %s" % ( pv_name, pvs_line ) )
						continue
					pv_wwid = get_pv_wwid_from_dm_minor( int( pv_name_match.group(1) ) )
					if not pv_wwid:
#						vi_errorAndLog_obj.vi_croak( "bad pv_name in pvscan : [%s] %s" % ( pv_name, pvs_line ) )
						vi_errorAndLog_obj.vi_print_err( "bad pv_name in pvscan : [%s] %s" % ( pv_name, pvs_line ) )
						continue

				vg_name = ''
				optional_vg_section = pvscan_line_match.group( 3 )
				if len( optional_vg_section ) > 3:
					optional_vg_match = optional_vg_cmpldRegEx.match( optional_vg_section )
					if optional_vg_match:
						vg_name = optional_vg_match.group( 1 )
			except:
#				vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
				vi_errorAndLog_obj.vi_print_err( "unexpected pvscan format '%s'\n%s" % ( pvs_line, traceback.format_exc() ) )
				continue

			pv_info         = PV_Info_OBJ()
			pv_info.mWWID   = pv_wwid
			pv_info.mUUID   = pv_uuid
			pv_info.mVGname = vg_name
#			vi_errorAndLog_obj.vi_print_dbg( 0, 'PV:', pv_info )
			pvInfo_uuid_DICT[ pv_uuid ] = pv_info

		del( pvs_lines_LIST )

#	vi_errorAndLog_obj.vi_print_dbg( 0, pvInfo_uuid_DICT )

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, 'vgs --noheadings --separator : --nosuffix --units b --options vg_name,vg_uuid,pv_uuid,vg_size,vg_free,snap_count' )
	del( stderr_LIST )
	vgs_lines_LIST = map( str.strip, stdout_LIST )
	del( stdout_LIST )
	vgs_line = ''
	try:
		for vgs_line in vgs_lines_LIST:
			if len( vgs_line ) < 1:
				continue
			vgs_line_LIST = vgs_line.strip().split( ':' )
			vg_name = vgs_line_LIST[ 0 ]
			if len( vg_name ) < 1:
				continue	# PV but no VG case

			vg_uuid = vgs_line_LIST[ 1 ]
			pv_uuid = vgs_line_LIST[ 2 ]
			pv_info = pvInfo_uuid_DICT.get( pv_uuid, None )
			if not pv_info:
				continue # new VG was just made accessible, right after PV info was scanned - skip it

			del( pvInfo_uuid_DICT[ pv_uuid ] )
			vg_info = vgInfo_uuid_DICT.get( vg_uuid, None )
			if not vg_info: # Use uuid as a name collision is bad yet not a fatal case
				vg_info						= VG_Info_OBJ()
				vg_info.mName				= vg_name
				vg_info.mUUID				= vg_uuid
				vg_info.mSizeInBytes		= int( vgs_line_LIST[ 3 ] )
				vg_info.mFreeSizeInBytes	= int( vgs_line_LIST[ 4 ] )
				vg_info.mPVinfo_wwid_DICT	= { pv_info.mWWID : pv_info }
				vg_info.mLVinfo_name_DICT	= {}
				vg_info.mNumOfLVsnapShots	= int( vgs_line_LIST[ 5 ] )
#				vi_errorAndLog_obj.vi_print_dbg( 0, 'VG:', vg_info )
				vgInfo_uuid_DICT[ vg_uuid ]   = vg_info
			else:
				if vg_info.mName != vg_name:
#					vi_errorAndLog_obj.vi_croak( "vg COLLISION '%s' vs '%s' for '%s'" % ( vg_name, vg_info.mName, vg_uuid ) )
					vi_errorAndLog_obj.vi_print_err( "vg COLLISION '%s' vs '%s' for '%s'" % ( vg_name, vg_info.mName, vg_uuid ) )
					continue
				if not vg_info.mPVinfo_wwid_DICT.has_key( pv_info.mWWID ):
					vg_info.mPVinfo_wwid_DICT[ pv_info.mWWID ] = pv_info
		del( vgs_lines_LIST )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "unexpected vgs format '%s'\n%s" % ( vgs_line, traceback.format_exc() ) )
		del( vgs_lines_LIST )

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, 'lvs --noheadings --separator : --nosuffix --units b --options lv_name,lv_uuid,vg_uuid,vg_name,lv_size,lv_attr,origin,snap_percent' )
	del( stderr_LIST )
	lvs_lines_LIST = map( str.strip, stdout_LIST )
	del( stdout_LIST )
	lvs_line = ''
	try:
		for lvs_line in lvs_lines_LIST:
			lvs_line_LIST = lvs_line.strip().split( ':' )
			lv_name     = lvs_line_LIST[ 0 ]
			lv_uuid     = lvs_line_LIST[ 1 ]
			vg_uuid     = lvs_line_LIST[ 2 ]
			vg_name     = lvs_line_LIST[ 3 ]
			vg_info     = vgInfo_uuid_DICT[ vg_uuid ]
			lvInfo_DICT = vg_info.mLVinfo_name_DICT
#			vi_errorAndLog_obj.vi_print_dbg( 0, 'LV:', lvInfo_DICT )
			lv_info = lvInfo_DICT.get( lv_name, None ) # Use name as a collision is more fatal within the VG
			if not lv_info:
				lv_info                 = LV_Info_OBJ()
				lv_info.mName           = lv_name
				lv_info.mUUID           = lv_uuid
				lv_info.mVGname         = vg_info.mName
				lv_info.mVGuuid         = vg_info.mUUID
				lv_info.mSizeInBytes    = int( lvs_line_LIST[ 4 ] )
				lv_info.mAttr           = lvs_line_LIST[ 5 ]
				lv_info.mOrigin         = lvs_line_LIST[ 6 ]
				lv_info.mSnapPercent    = lvs_line_LIST[ 7 ]
#				vi_errorAndLog_obj.vi_print_dbg( 0, lv_info )

				lvInfo_DICT[ lv_name ]  = lv_info
			else:
				if lv_info.mUUID != lv_uuid:
#					vi_errorAndLog_obj.vi_croak( "lv COLLISION '%s' vs '%s' for '%s'" % ( lv_uuid, lv_info.mUUID, lv_name ) )
					vi_errorAndLog_obj.n_print_err( "lv COLLISION '%s' vs '%s' for '%s'" % ( lv_uuid, lv_info.mUUID, lv_name ) )
					continue
		del( lvs_lines_LIST )
	except:
#		vi_errorAndLog_obj.vi_croak_exception( traceback.format_exc() )
		vi_errorAndLog_obj.vi_print_err( "unexpected lvs format '%s'\n%s" % ( lvs_line, traceback.format_exc() ) )
		del( lvs_lines_LIST )

	if 0:
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )
		vi_errorAndLog_obj.vi_print_dbg( 0, pvInfo_uuid_DICT )
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )
		vi_errorAndLog_obj.vi_print_dbg( 0, vgInfo_uuid_DICT )
		vi_errorAndLog_obj.vi_print_dbg( 0, '' )

	return ( pvInfo_uuid_DICT, vgInfo_uuid_DICT )

####################################################################################################
def get_lvm_info( vi_errorAndLog_obj, refresh ):
	"""
	This routine
	"""
#
	global global__cached__pvInfo_uuid_DICT
	global global__cached__vgInfo_uuid_DICT
	if not refresh and global__cached__pvInfo_uuid_DICT:
		return ( global__cached__pvInfo_uuid_DICT, global__cached__vgInfo_uuid_DICT )

	pvInfo_uuid_DICT, vgInfo_uuid_DICT	= add_lvm_info( vi_errorAndLog_obj, True, {}, {} )

	global__cached__pvInfo_uuid_DICT 	= pvInfo_uuid_DICT
	global__cached__vgInfo_uuid_DICT 	= vgInfo_uuid_DICT

	return ( global__cached__pvInfo_uuid_DICT, global__cached__vgInfo_uuid_DICT )

####################################################################################################
def get_basic_vg_info( vi_errorAndLog_obj, refresh ):
	"""
	This routine
	"""
#

	global global__cached__basicVGinfo_uuid_DICT
	if not refresh and global__cached__basicVGinfo_uuid_DICT:
		return global__cached__basicVGinfo_uuid_DICT

	basicVGinfo_uuid_DICT	= {}

	stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, 'vgs --noheadings --separator : --nosuffix --units b --options vg_name,vg_uuid,vg_size,vg_free,pv_count,lv_count,snap_count' )
	del stderr_LIST
	for vgs_line in stdout_LIST:
		if len( vgs_line ) < 1:
			continue

		vgs_line_LIST					= vgs_line.strip().split( ':' )

		basic_vg_info					= Basic_VG_Info_OBJ()
		basic_vg_info.mName				= vgs_line_LIST[ 0 ]
		basic_vg_info.mUUID				= vgs_line_LIST[ 1 ]
		basic_vg_info.mSizeInBytes		= int( vgs_line_LIST[ 2 ] )
		basic_vg_info.mFreeSizeInBytes	= int( vgs_line_LIST[ 3 ] )
		basic_vg_info.mNumOfPVs			= int( vgs_line_LIST[ 4 ] )
		basic_vg_info.mNumOfLVs			= int( vgs_line_LIST[ 5 ] )
		basic_vg_info.mNumOfLVsnapShots	= int( vgs_line_LIST[ 6 ] )

		basicVGinfo_uuid_DICT[ basic_vg_info.mUUID ]   = basic_vg_info

	global__cached__basicVGinfo_uuid_DICT	= basicVGinfo_uuid_DICT

	return global__cached__basicVGinfo_uuid_DICT

####################################################################################################
def is_vdisk_lv_name( lv_name ):
	"""
	This routine

	Device is named as "/dev/<VG's UUID>/<LV's UUID>"

	UUID format:
	                  |> Type (see below)
	                  |
	VI's prefix    	  ||> Version
	 __|__            ||
	|     |           ||
	6000F4B00000000000XY0111b8e4667f   <- 32 characters wide

	Types:
	LvmLocalGroup         1
	LvmNetworkGroup       2
	LvmVirtualDiskGroup   3
	LvmLogicalDiskGroup   4
	LvmPartition          5
	LogicalDisk           6
	VirtualDisk           7
	"""
#
	if len( lv_name ) != 32:
		return False

	lv_name   = lv_name.upper()

	vi_prefix = lv_name[ 0 : 7 ]
	if vi_prefix != '6000F4B':
		return False

	try:
		lv_lvm_type    = int( lv_name[ 19 : 20 ], 16 )
		lv_lvm_version = int( lv_name[ 20 : 21 ], 16 )
	except:
		return False

	if lv_lvm_version == 0:
		return True # Really UNknown as the feature of typing the objects was not present in version zero

	if lv_lvm_type != 3:
		return False

	return True

####################################################################################################
def do_dom0_memory_safe_vgchange( vi_errorAndLog_obj, enableLVvsDisableLV, vgname_LIST, refresh ):
	"""
	This routine 
	"""
#
	if enableLVvsDisableLV:
		avail_arg	= 'ly'
	else:
		avail_arg	= 'ln'

	basicVGinfo_uuid_DICT	= get_basic_vg_info( vi_errorAndLog_obj, refresh )

	for vg_uuid, basic_vg_info in basicVGinfo_uuid_DICT.iteritems():

		if vgname_LIST != None and not basic_vg_info.mName in vgname_LIST:
			continue

		if basic_vg_info.mNumOfLVs < 50:

			cmd_LIST	= [ 'vgchange', '--available', avail_arg, basic_vg_info.mName ]

			stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST )
			if len( stdout_LIST ):
				vi_errorAndLog_obj.vi_print_log( ''.join( stdout_LIST ), None )
			if len( stderr_LIST ):
				vi_errorAndLog_obj.vi_print_log( ''.join( stderr_LIST ), None )

			continue

		dom0memCushGuardInKB	= ( basic_vg_info.mNumOfLVs / 50 ) * 10240	# Found via experimentation

		cmd_LIST	= [ 'dom0memcontroller', '--ensureDom0memoryCushion', '--increaseDom0memorySizeKB', str(dom0memCushGuardInKB), "--subCmd", 'vgchange', '--available', avail_arg, basic_vg_info.mName ]

		vi_errorAndLog_obj.vi_print_dbg( 5, "VG: %s %d %d %d" % ( basic_vg_info.mName, basic_vg_info.mNumOfPVs, basic_vg_info.mNumOfLVs, basic_vg_info.mNumOfLVsnapShots ) )
		vi_errorAndLog_obj.vi_print_dbg( 3, 'dom0memcontroller --dumpMemInfo', ' '.join( cmd_LIST[1:] ) )

		stdout_LIST, stderr_LIST, exitStatus, deltaTime = st_utils.vi_exe_sync( vi_errorAndLog_obj, cmd_LIST )

		subCmdBeg_cmpldRegEx	= re.compile( '^BEG:================================================================$' )
		subCmdEnd_cmpldRegEx	= re.compile( '^END:================================================================\[(\d+)\|(\w+)\]$' )
		subCmdOutput_LIST		= []
		subCmdExitStatus		= None

		keepLine				= False
		for line in stdout_LIST:
			if subCmdBeg_cmpldRegEx.match( line ):
				keepLine		= True
				continue
			subCmdEnd_match	= subCmdEnd_cmpldRegEx.match( line )
			if not subCmdEnd_match:
				if keepLine:
					subCmdOutput_LIST.append( line.rstrip() )
				continue
			subCmdExitStatus	= int(subCmdEnd_match.group(1))
			break

		if subCmdExitStatus == None:
			vi_errorAndLog_obj.vi_print_log( 'ERROR:' )
			vi_errorAndLog_obj.vi_print_log( ' '.join( cmd_LIST ) )
			if len( stdout_LIST ):
				vi_errorAndLog_obj.vi_print_log( ''.join( stdout_LIST ), None )
			if len( stderr_LIST ):
				vi_errorAndLog_obj.vi_print_log( ''.join( stderr_LIST ), None )
		else:
			vi_errorAndLog_obj.vi_print_log( '\n'.join( subCmdOutput_LIST ) )

####################################################################################################
# END
